import json
import textwrap


class Vulnerability:
    def __init__(self, kind=None, description=None, transactions=None):
        self.type = kind
        self.description = description
        self.transactions = transactions
        self.confirmed = False

    def __str__(self):
        return textwrap.dedent(
            """\
            Type = {type}
            Description = {description}
            Transactions = {transactions}
            """.format(
                type=self.type,
                description=self.description,
                transactions=self.transactions,
                sort_keys=True,
                indent=4,
            )
        )

    def json(self):
        return json.dumps(self)



"""This module contains a basic Ethereum RPC client.

It was changed to allow Infura connections.

This code is adapted from: https://github.com/ConsenSys/mythril
"""
import json
import logging

import requests
from requests.adapters import HTTPAdapter
from requests.exceptions import ConnectionError

from mythril.ethereum.interface.rpc.base_client import BaseClient
from mythril.ethereum.interface.rpc.exceptions import (
    BadJsonError,
    BadResponseError,
    BadStatusCodeError,
    ConnectionError as MythrilConnectionError,
)

log = logging.getLogger(__name__)

MAX_RETRIES = 3
JSON_MEDIA_TYPE = "application/json"


class EthJsonRpc(BaseClient):
    """Ethereum JSON-RPC client class."""

    def __init__(self, url=None):
        """

        :param url:
        """
        self.url = url
        self.session = requests.Session()
        self.session.mount(self.url, HTTPAdapter(max_retries=MAX_RETRIES))

    def _call(self, method, params=None, _id=1):
        """

        :param method:
        :param params:
        :param _id:
        :return:
        """
        params = params or []
        data = {"jsonrpc": "2.0", "method": method, "params": params, "id": _id}
        headers = {"Content-Type": JSON_MEDIA_TYPE}
        log.debug("rpc send: %s" % json.dumps(data))
        try:
            r = self.session.post(self.url, headers=headers, data=json.dumps(data))
        except ConnectionError:
            raise MythrilConnectionError
        if r.status_code / 100 != 2:
            raise BadStatusCodeError(r.status_code)
        try:
            response = r.json()
            log.debug("rpc response: %s" % response)
        except ValueError:
            raise BadJsonError(r.text)
        try:
            return response["result"]
        except KeyError:
            raise BadResponseError(response)

    def close(self):
        """Close the RPC client's session."""
        self.session.close()



from setuptools import setup, find_packages
from karl.version import __version__
from setuptools.command.install import install
import sys
import os
import shutil
import pathlib
import atexit


def read_file(fname):
    """
    return file contents
    :param fname: path relative to setup.py
    :return: file contents
    """
    with open(os.path.join(os.path.dirname(__file__), fname), "r") as fd:
        return fd.read()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""

    description = "verify that the git tag matches our version"

    def run(self):
        tag = os.getenv("CIRCLE_TAG")

        if tag != __version__:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, __version__
            )
            sys.exit(info)


def copySignatures():
    """Check if signatures.db is already installed and copy it in the home dir if not."""
    mythril_path = os.path.join(pathlib.Path.home(), ".mythril")
    signatures_path = os.path.join(mythril_path, "signatures.db")
    if not os.path.exists(mythril_path):
        print("Mythril db folder does not exist, creating...")
        try:
            os.mkdir(mythril_path)
        except OSError:
            print("Could not create Mythril db folder")
            print("Manually create ~/.mythril/signatures.db")

    if not os.path.exists(signatures_path):
        print("Signatures does not exist, creating...")
        try:
            shutil.copyfile("./signatures/signatures.db", signatures_path)
        except Exception as e:
            print(sys.exc_info()[0])
            print(type(e))  # the exception instance
            print(e.args)  # arguments stored in .args
            print(e)

            print(
                "Could not copy signatures.db file to {signatures_dest}".format(
                    signatures_dest=signatures_path
                )
            )
            print("Manually create ~/.mythril/signatures.db")


class PostInstallCommand(install):
    """Post-installation for installation mode."""

    def __init__(self, *args, **kwargs):
        super(PostInstallCommand, self).__init__(*args, **kwargs)
        atexit.register(copySignatures)


setup(
    name="karl",
    version=__version__,
    license="MIT",
    author="Daniel Luca",
    author_email="daniel.luca@consensys.net",
    long_description=read_file("Readme.md") if os.path.isfile("Readme.md") else "",
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    package_data={"karl": ["signatures/signatures.db"]},
    include_package_data=True,
    install_requires=read_file("requirements.txt").split("\n"),
    entry_points={"console_scripts": ["karl=karl.interfaces.cli:main"]},
    credits=["Bernhard Mueller <bernhard.mueller@mithril.ai>"],
    cmdclass={"verify": VerifyVersionCommand, "install": PostInstallCommand},
)
